#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
from os.path import dirname, realpath
import sys
from .ui import MainWindow, resource_rc
from urllib.request import urlopen
# from PyQt5.QtGui import QApplication, QIcon
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *

def main():
    app = QApplication(sys.argv)

    #UI Design style
    app.setStyle("Fusion")
    palette = QPalette()
    palette.setColor(QPalette.Window, QColor(250, 250, 250))
    palette.setColor(QPalette.WindowText, Qt.black)
    palette.setColor(QPalette.Base, QColor(233, 233, 233))
    palette.setColor(QPalette.AlternateBase, QColor(220, 220, 220))
    palette.setColor(QPalette.ToolTipBase, Qt.white)
    palette.setColor(QPalette.ToolTipText, Qt.black)
    palette.setColor(QPalette.Text, Qt.black)
    palette.setColor(QPalette.Button, QColor(220, 220, 220))
    palette.setColor(QPalette.ButtonText, Qt.black)
    palette.setColor(QPalette.BrightText, Qt.red)
    palette.setColor(QPalette.Link, QColor(42, 130, 218))
    palette.setColor(QPalette.Highlight, QColor(52, 140, 228))
    palette.setColor(QPalette.HighlightedText, Qt.black)
    app.setPalette(palette)

    translator = QTranslator()
    locale = QLocale.system().name()[:2]
    if getattr(sys, 'frozen', False):
        # frozen
        dir_ = dirname(sys.executable)
    else:
        # unfrozen
        dir_ = dirname(realpath(__file__))
        if locale == "fr":
            translator.load('ui/i18n/tr_fr', dir_)
        app.installTranslator(translator)
        win = MainWindow()
        app.setWindowIcon(QIcon(':/white_renamer48.png'))
        # MainWidget.SizeCalculator(win)
        win.show()
        app.exec_()



#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.

from . import action_manager
from .file_system import FileDescriptor, Controller, FileSystemTreeNode, FileSystem



#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
import sys
import os
import os.path
import subprocess
import webbrowser
import urllib.request
import whiterenamer

from PyQt5.QtCore import pyqtSlot, QSize
from PyQt5.QtWidgets import QMainWindow, QAction, QActionGroup, QLineEdit, QWidget, QSizePolicy, QFileDialog, QMessageBox
from PyQt5.QtGui import QIcon

from . import MainWidget, resource_rc
from ..model import FileSystem


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self)
        self.setWindowTitle('White Renamer')
        # self.setWindowIcon(QIcon(':/white_renamer48.png'))
        self.setUnifiedTitleAndToolBarOnMac(True)
        self.directory = None
        self.use_subfolder = False
        self.filtered_collection = None
        self.files_collection = None
        self.resize(1000, 800)
        self.showMaximized()
        self.name_filter = ''
        self.files_type = ['*.*']
        self.reverse_order = False
        self.sorting_criteria = "name"

        #CREATE THE ACTIONS
        self.action_open = QAction(self.tr('&Open'), self)
        self.action_open = self.edit_action(self.action_open, self.open_directory_dialog_click, None, 'ctrl+o', "new_icon.png" ,self.tr('Open directory.'))
        self.action_exit = QAction(self.tr('&Exit'), self)
        self.action_exit = self.edit_action(self.action_exit, self.close, None,'ctrl+q', "exit_icon.png", self.tr('Exit the application.'))
        self.action_help = QAction(self.tr('&Help'), self)
        self.action_help = self.edit_action(self.action_help, self.help_click, None, 'ctrl+h', 'help_icon.png', self.tr('Show help page.'))
        self.action_about = QAction(self.tr('&About'), self)
        self.action_about = self.edit_action(self.action_about, self.about_box_click, None, None, None,self.tr('About Box.'))


        self.action_check_update = QAction(self.tr('&Check Updates'), self)
        self.action_check_update = self.edit_action(self.action_check_update, self.check_update_click, None, None, None,self.tr('Check for updates.'))

        self.action_recursion = QAction(self.tr('Show Subdirectories'), self)
        self.action_recursion = self.edit_action(self.action_recursion, self.recursion_click, bool, None, "subdirectory_icon.png",self.tr('Rename subdirectories recursively.'))
        self.action_recursion.setCheckable(True)
        self.action_hide = QAction(self.tr('Show Hidden Files'), self)
        self.action_hide = self.edit_action(self.action_hide, self.hide_files_click, bool, 'ctrl+h', "hidden_icon.png",self.tr('Show hidden files.'))
        self.action_hide.setCheckable(True)
        self.action_rename = QAction(self.tr('&Rename'), self)
        self.action_rename = self.edit_action(self.action_rename, self.rename_click, None, 'ctrl+enter', "run_icon.png",self.tr('Rename the files/folders.'))
        self.action_undo = QAction(self.tr('Undo'), self)
        self.action_undo = self.edit_action(self.action_undo, self.undo_click, None, 'ctrl+z', "undo_icon.png",self.tr('Undo the previous renaming.'))
        self.action_reverse_sorting = QAction(self.tr('Reverse'), self)
        self.action_reverse_sorting.setCheckable(True)
        self.action_reverse_sorting = self.edit_action(self.action_reverse_sorting, self.reverse_sorting_click, bool, None, "order_icon.png",self.tr('Reverse the sorting order.'))
        self.action_name_sorting = QAction(self.tr('By Name'), self)
        self.action_name_sorting.setCheckable(True)
        self.action_name_sorting = self.edit_action(self.action_name_sorting, self.name_sorting_click, None, None, None,self.tr('Sort the files/folders by name.'))
        self.action_name_sorting.setChecked(True)
        self.action_size_sorting = QAction(self.tr('By Size'), self)
        self.action_size_sorting.setCheckable(True)
        self.action_size_sorting = self.edit_action(self.action_size_sorting, self.size_sorting_click, None, None, None,self.tr('Sort the files/folders by size.'))
        self.action_modified_date_sorting = QAction(self.tr('By Modified Date'), self)
        self.action_modified_date_sorting.setCheckable(True)
        self.action_modified_date_sorting = self.edit_action(self.action_modified_date_sorting, self.modified_date_sorting_click, None, None, None,self.tr('Sort the files/folders by modified date.'))
        self.action_creation_date_sorting = QAction(self.tr('By Creation Date'), self)
        self.action_creation_date_sorting.setCheckable(True)
        self.action_creation_date_sorting = self.edit_action(self.action_creation_date_sorting, self.creation_date_sorting_click, None, None, None,self.tr('Sort the files/folders by creation date.'))
        sort_group = QActionGroup(self)
        sort_group.addAction(self.action_name_sorting)
        sort_group.addAction(self.action_size_sorting)
        sort_group.addAction(self.action_modified_date_sorting)
        sort_group.addAction(self.action_creation_date_sorting)

        filterInput = QLineEdit()
        filterInput.setPlaceholderText(self.tr("Filter Files..."))
        filterInput.setMaximumWidth(150)
        filterInput.textChanged[str].connect(self.get_name_filter)
        self.action_files_only = QAction(self.tr('Files Only'), self)
        self.action_files_only.setCheckable(True)
        self.action_files_only.setChecked(True)
        self.action_files_only = self.edit_action(self.action_files_only, self.files_only_click, None, None, "file_icon.png",self.tr('Rename only files.'))
        self.action_folders_only = QAction(self.tr('Folders Only'), self)
        self.action_folders_only.setCheckable(True)
        self.action_folders_only = self.edit_action(self.action_folders_only, self.folders_only_click, None, None, "folder_icon.png",self.tr('Rename only folders.'))

        node_type_selector = QActionGroup(self)
        node_type_selector.setObjectName('selector')
        node_type_selector.addAction(self.action_files_only)
        node_type_selector.addAction(self.action_folders_only)

        file_type = QActionGroup(self)
        self.action_all_files = QAction(self.tr("All"),self)
        self.action_all_files = self.edit_action(self.action_all_files, self.all_files_click, None, None, "all_files_icon.png",self.tr('Rename files of any kind.'))
        self.action_all_files.setCheckable(True)
        self.action_all_files.setChecked(True)
        self.action_music_files = QAction(self.tr("Music"),self)
        self.action_music_files = self.edit_action(self.action_music_files, self.music_files_click, None, None, "music_icon.png",self.tr('Rename only music files.'))
        self.action_music_files.setCheckable(True)
        self.action_image_files = QAction(self.tr("Images"),self)
        self.action_image_files = self.edit_action(self.action_image_files, self.image_files_click, None, None, "images_icon.png",self.tr('Rename only image files.'))
        self.action_image_files.setCheckable(True)
        file_type.addAction(self.action_all_files)
        file_type.addAction(self.action_image_files)
        file_type.addAction(self.action_music_files)
        # CREATE THE MENU BAR
        menubar = self.menuBar()
        #FILE
        menu_file = menubar.addMenu(self.tr('&File'))
        menu_file.addAction(self.action_open)
        menu_file.addSeparator()
        menu_file.addAction(self.action_hide)
        menu_file.addAction(self.action_recursion)
        menu_file.addSeparator()
        menu_file.addAction(self.action_exit)
        #EDIT
        menu_edit = menubar.addMenu(self.tr('&Sort'))
        menu_edit.addAction(self.action_name_sorting)
        menu_edit.addAction(self.action_size_sorting)
        menu_edit.addAction(self.action_creation_date_sorting)
        menu_edit.addAction(self.action_modified_date_sorting)
        menu_edit.addSeparator()
        menu_edit.addAction(self.action_reverse_sorting)
        menu_filter = menubar.addMenu(self.tr('&Filter'))
        menu_filter.addAction(self.action_files_only)
        menu_filter.addAction(self.action_folders_only)
        menu_filter.addSeparator()
        menu_filter.addAction(self.action_all_files)
        menu_filter.addAction(self.action_image_files)
        menu_filter.addAction(self.action_music_files)
        #TOOL
        menu_tool = menubar.addMenu(self.tr('&Tool'))
        menu_tool.addAction(self.action_rename)
        menu_tool.addAction(self.action_undo)
        #HELP
        menu_help = menubar.addMenu(self.tr('&Help'))
        menu_help.addAction(self.action_help)
        menu_help.addAction(self.action_about)
        menu_help.addAction(self.action_check_update)

        self.main_toolbar = self.addToolBar('main_toolbar')
        self.main_toolbar.addAction(self.action_open)
        self.main_toolbar.addSeparator()
        self.main_toolbar.addAction(self.action_hide)
        self.main_toolbar.addAction(self.action_recursion)
        self.main_toolbar.addSeparator()
        self.main_toolbar.addAction(self.action_files_only)
        self.main_toolbar.addAction(self.action_folders_only)
        self.main_toolbar.addSeparator()
        self.main_toolbar.addAction(self.action_all_files)
        self.main_toolbar.addAction(self.action_image_files)
        self.main_toolbar.addAction(self.action_music_files)
        self.main_toolbar.addWidget(filterInput)
        self.main_toolbar.addSeparator()
        self.main_toolbar.addAction(self.action_rename)
        self.main_toolbar.addAction(self.action_undo)
        self.main_toolbar.setIconSize(QSize(16,16))

        empty = QWidget();
        empty.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred)
        self.main_toolbar.addWidget(empty)
        self.main_toolbar.addAction(self.action_help)
        # create the status bar
        self.statusBar()
        self.main_widget = MainWidget()
        self.setCentralWidget(self.main_widget)

    def get_main_widget(self):
        return self.main_widget


    def edit_action(self, action, slot=None, type=None, shortcut=None, icon=None, tip=None):
        '''This method adds to action: icon, shortcut, ToolTip,\
        StatusTip and can connect triggered action to pyqtSlot '''
        if icon is not None:
            action.setIcon(QIcon(":/{0}".format(icon)))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            if type is not None:
                action.triggered[type].connect(slot)
            else:
                action.triggered.connect(slot)

        return action

    @pyqtSlot()
    def get_name_filter(self, value):
        self.name_filter = value
        self.reset_files_collection()

    @pyqtSlot()
    def help_click(self):
        '''Read and display a help file- currently the README.txt.'''
        if getattr(sys, 'frozen', False): # frozen
            dir_ = os.path.dirname(sys.executable)
            filepath = os.path.join(dir_, "Documentation.pdf")
        else: # unfrozen
            dir_ = os.path.dirname(os.path.realpath(__file__))
            filepath = os.path.join(dir_, "..","doc", "Documentation.pdf")

        if sys.platform.startswith('darwin'):
            subprocess.call(('open', filepath))
        elif os.name == 'nt':
            os.startfile(filepath)
        elif os.name == 'posix':
            subprocess.call(('xdg-open', filepath))

    @pyqtSlot()
    def music_files_click(self):
        self.files_type = ['.flac', '.mp3', '.m4a', '.ogg', '.wma', '.m3a', '.mp4']
        self.reset_files_collection()

    @pyqtSlot()
    def image_files_click(self):
        self.files_type = ['.jpg', '.jpeg', '.tif', '.png', '.gif', '.bmp', '.eps', '.im', '.jfif', '.j2p', '.jpx', '.pcx', '.ico', '.icns', '.psd', '.nef', 'cr2', 'pef']
        self.reset_files_collection()

    @pyqtSlot()
    def all_files_click(self):
        self.files_type = ['*.*']
        self.reset_files_collection()

    @pyqtSlot()
    def files_only_click(self):
        self.files_type = ['*.*']
        self.action_all_files.setChecked(True)
        self.action_all_files.setEnabled(True)
        self.action_image_files.setEnabled(True)
        self.action_music_files.setEnabled(True)
        self.main_widget.is_file(True)
        self.reset_files_collection()

    @pyqtSlot()
    def folders_only_click(self):
        self.files_type = ["folders"]
        self.action_all_files.setChecked(True)
        self.action_all_files.setEnabled(False)
        self.action_image_files.setEnabled(False)
        self.action_music_files.setEnabled(False)
        self.main_widget.is_file(False)
        self.reset_files_collection()


    @pyqtSlot()
    def about_box_click(self):
        '''Popup a box with about message.'''
        QMessageBox.about(self, "About WhiteRenamer",
                """<b>White Renamer</b> 
                <p>Copyright &copy; 2015 Pierre BLANC.</p>
                <p>email : <a href="mailto:pierrecnalb@mailbox.org">pierrecnalb@mailbox.org</a></p>
                <p> White Renamer is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.</p>

<p>White Renamer is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; See the GNU General Public License for more details.</p>
                 """ )
    @pyqtSlot()
    def check_update_click(self):
        try:
            code_online = urllib.request.urlopen("https://raw.githubusercontent.com/pierrecnalb/White-Renamer/master/whiterenamer/version.txt").read().splitlines()
            version_online = code_online[0].decode().split('.')
            version_online = str(whiterenamer.Version(version_online[0], version_online[1], version_online[2]))
            print(version_online)
            self.update_message(str(whiterenamer.__version__), version_online)
        except:
            raise Exception("Unable to retrieve the new software version from the server. Please try later.")

    @pyqtSlot(bool)
    def recursion_click(self, value):
        self.use_subfolder = value
        if self.directory is None:
            return
        self.reset_files_collection()

    @pyqtSlot(bool)
    def hide_files_click(self, value):
        self.show_hidden_files = value
        self.reset_files_collection()

    def are_hidden_files_shown(self):
        return self.action_hide.isChecked()

    @pyqtSlot()
    def open_directory_dialog_click(self):
        """Opens a dialog to allow user to choose a directory """
        flags = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
        try:
            self.directory = QFileDialog.getExistingDirectory(self,self.tr("Select Directory"), os.getcwd(), flags)
            self.reset_files_collection()
        except Exception as e:
            print(str(e))
            msg_box = QMessageBox.warning(self, "Invalid directory", "Please select a valid directory." )

    @pyqtSlot(bool)
    def reverse_sorting_click(self, value):
        self.reverse_order = value
        self.reset_files_collection()

    @pyqtSlot()
    def name_sorting_click(self):
        self.sorting_criteria = "name"
        self.reset_files_collection()
    @pyqtSlot()
    def size_sorting_click(self):
        self.sorting_criteria = "size"
        self.reset_files_collection()
    @pyqtSlot()
    def creation_date_sorting_click(self):
        self.sorting_criteria = "creation_date"
        self.reset_files_collection()
    @pyqtSlot()
    def modified_date_sorting_click(self):
        self.sorting_criteria = "modified_date"
        self.reset_files_collection()

    def reset_files_collection(self):
        if(self.directory is None):
            return
        self.files_system = FileSystem(self.directory, self.use_subfolder)
        self.files_system_view = self.files_system.generate_files_system_view(self.are_hidden_files_shown(), self.files_type, self.name_filter, self.sorting_criteria, self.reverse_order)
        self.main_widget.set_filtered_files(self.files_system_view)

    @pyqtSlot()
    def rename_click(self):
        self.main_widget.rename()
        self.reset_files_collection()

    @pyqtSlot()
    def undo_click(self):
        self.main_widget.undo()

    def update_message(self, version, new_version):
        if(version == new_version):
            msg_box = QMessageBox.information(self, "Version", 
                """<p>Your version of whiterenamer is up to date.</p>
                """.format(version, new_version), QMessageBox.Ok)

        else:
            msg_box = QMessageBox.information(self, "Update available", 
                                              """
                <b>New version available</b> 
                <p>You are running an old version of White Renamer (v{0}).</p>
                <p>A newer version is available (v{1}). Do you want to download it ?</p>
                """.format(version, new_version), QMessageBox.No, QMessageBox.Yes)
            if msg_box == QMessageBox.No:
                pass
            if msg_box == QMessageBox.Yes:
                new = 2 # open in a new tab, if possible
                # open a public URL, in this case, the webbrowser docs
                url = "https://github.com/pierrecnalb/WhiteRenamer-builds"
                webbrowser.open(url,new=new)
                # Save was clicked
                # elif ret == QMessageBox.Discard:
                # Don't save was clicked



#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
from os import listdir
from os.path import getsize, join, isdir, getmtime, getctime, abspath, relpath, split, splitext, exists
from shutil import move
from copy import deepcopy
from io import open
from uuid import uuid4
from ..ui import FileSystemView

class FileDescriptor(object):
    """
    Group information related to the input files.
    Parameters:
        --basename: string that represents the name of the file or folder.
        --is_folder: boolean that tells if the current FileDescriptor is a directory or a file.
    """
    def __init__(self, basename, is_folder):
        self._basename = basename
        self.is_folder = is_folder
        if (self.is_folder is False):
            (self._filename, self._extension) = splitext(self._basename)
            self._extension = self._extension[1:] #remove dot
            self._foldername = ""
        else:
            self._filename = ""
            self._foldername = self._basename
            self._extension = ""
        self._prefix = ""
        self._suffix = ""

    def __repr__(self):
        """override string representation of the class"""
        return self._basename

    def is_folder(self):
        return self.is_folder

    @property
    def basename(self):
        self.update_basename()
        return self._basename

    @basename.setter
    def basename(self, value):
        self._basename = value

    @property
    def foldername(self):
        return self._foldername

    @foldername.setter
    def foldername(self, value):
        self._foldername = value
        self.update_basename()

    @property
    def suffix(self):
        return self._suffix

    @suffix.setter
    def suffix(self, value):
        self._suffix = value
        self.update_basename()

    @property
    def prefix(self):
        return self._prefix

    @prefix.setter
    def prefix(self, value):
        self._prefix = value
        self.update_basename()

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, value):
        self._filename = value
        self.update_basename()

    @property
    def extension(self):
        return self._extension

    @extension.setter
    def extension(self, value):
        self._extension = value
        self.update_basename()

    def update_basename(self):
        if self.is_folder is True:
            self._basename = self._prefix + self._foldername + self._suffix
        else:
            self._basename = self._prefix + self._filename + self._suffix + "." + self._extension

class FileSystemTreeNode(object):
    """
    Contains the original, modified and backup FileDescriptor for a given node of the specified directory.
    The structure of the system tree node is reproduced by adding children for each subdirectory.

    Parameters:
        --root_path: string that represents the whole path of the current file or directory.
        --parent: the FileSystemTreeNode parent of the current FileSystemTreeNode.
        --original_filedescriptor: represents the current file or directory basename.
        --is_folder: boolean that tells if the current FileSystemTreeNode is a directory or a file.
        --size: size of the file in bytes.
        --modified_date: time of last modification.
        --created_date: time of creation.
    """
    def __init__(self, root_path, parent, original_filedescriptor, is_folder, is_hidden, size, modified_date, created_date):
        self.children = []
        self._root_path = root_path
        self._original_filedescriptor = original_filedescriptor
        self._modified_filedescriptor = self._original_filedescriptor
        self._parent = parent
        self._backup_filedescriptor = self._original_filedescriptor
        self._is_folder = is_folder
        self._is_hidden = is_hidden
        self._size = size
        self._modified_date = modified_date
        self._created_date = created_date
        self._has_children = False
        self._rank = -1

    def __repr__(self):
        return self.get_original_relative_path()

    @property
    def rank(self):
        """Give the position of the file according to the sorting criteria."""
        return self._rank

    @rank.setter
    def rank(self, value):
        self._rank = value
        self.files_system_tree_node.rank = value

    def add_children(self, file_system_tree_node):
        self.children.append(file_system_tree_node)
        return file_system_tree_node

    def get_children(self):
        return self.children

    def find_child_by_path(self, path):
        for child in self.get_children():
            if child.get_original_path() in path:
                return child

    def get_parent(self):
        return self._parent

    def get_original_relative_path(self):
        if self._parent != None:
            return join(self._parent.get_original_relative_path(), self._original_filedescriptor.basename)
        else:
            return self._original_filedescriptor.basename

    def get_modified_relative_path(self):
        if self._parent != None:
            return join(self._parent.get_modified_relative_path(), self._modified_filedescriptor.basename)
        else:
            return self._modified_filedescriptor.basename

    def get_backup_relative_path(self):
        if self._parent != None:
            return join(self._parent.get_backup_relative_path(), self._backup_filedescriptor.basename)
        else:
            return self._backup_filedescriptor.basename

    def get_original_path(self):
        return join(self._root_path, self.get_original_relative_path())

    def get_modified_path(self):
        return join(self._root_path, self.get_modified_relative_path())

    def get_backup_path(self):
        return join(self._root_path, self.get_backup_relative_path())
    @property
    def size(self):
        return self._size
    @property
    def modified_date(self):
        return self._modified_date
    @property
    def created_date(self):
        return self._created_date

    @property
    def is_folder(self):
        return self._is_folder

    @property
    def original_filedescriptor(self):
        return self._original_filedescriptor

    @original_filedescriptor.setter
    def original_filedescriptor(self, value):
        self._original_filedescriptor = value

    @property
    def modified_filedescriptor(self):
        return self._modified_filedescriptor

    @modified_filedescriptor.setter
    def modified_filedescriptor(self, value):
        self._modified_filedescriptor = value

    @property
    def backup_filedescriptor(self):
        return self._backup_filedescriptor

    @backup_filedescriptor.setter
    def backup_filedescriptor(self, value):
        self._backup_filedescriptor = value

    @property
    def is_hidden(self):
        return self._is_hidden

    @property
    def has_children(self):
        return self._has_children

    @has_children.setter
    def has_children(self, value):
        self._has_children = value

    def match_files_type(self, files_type):
        if (files_type == ['*.*']):
            return True
        if(files_type == ["folders"]):
            if(self.is_folder is True):
                return True
            else:
                return False
        elif(self.is_folder is False):
            name = self.original_filedescriptor.extension.lower()
            for ext in files_type:
                if (name in ext):
                    return True
        return False

    def match_name_filter(self, name_filter):
        if(name_filter == ""):
            return True
        if(self.is_folder is False):
            name = str(self.original_filedescriptor.filename).lower()
        else:
            name = str(self.original_filedescriptor.foldername).lower()
        if (name_filter in name):
            return True
        return False

class FileSystem(object):
    """
    Contains all the FileSystemTreeNodes representing the files system structure with or without the subdirectories, starting from the input path.
    Parameters:
        --input_path: string that represents the root directory to start the files collection from.
        --use_subdirectory: boolean that tells to look over the subdirectories recursively or not.
    """

    def __init__(self, input_path, use_subdirectory):
        self.input_path = input_path
        (self.root_folder, basename)=split(self.input_path)
        self.use_subdirectory = use_subdirectory
        root_filedescriptor = FileDescriptor(basename, True)
        self.root_tree_node = FileSystemTreeNode(self.root_folder, None, root_filedescriptor, True, False,0,0,0 )
        self.scan(self.root_tree_node)
        self.renamed_files_list = []

    def scan(self, tree_node):
        """Creates the files system structure with FileSystemTreeNode."""
        path = self.get_full_path(tree_node.get_original_relative_path())
        children = listdir(path)
        for child in children:
            size = getsize(join(path,child))#return 0 when folders.
            modified_date = getmtime(join(path,child))
            created_date = getctime(join(path,child))
            if isdir(join(path,child)):
                file_system_child_node = self.add_folder(tree_node, child, size, modified_date, created_date)
                if (not self.use_subdirectory):
                    continue
                else:
                    self.scan(file_system_child_node)
            else:
                self.add_file(tree_node, child, size, modified_date, created_date)

    def add_file(self, tree_node, child, size, modified_date, created_date):
        file_system_child_node = FileSystemTreeNode(self.root_folder,tree_node, FileDescriptor(child, False), False, child.startswith('.'), size, modified_date, created_date)
        tree_node.has_children = True
        tree_node.add_children(file_system_child_node)

    def add_folder(self, tree_node, child, size, modified_date, created_date):
        file_system_child_node = FileSystemTreeNode(self.root_folder,tree_node, FileDescriptor(child, True), True, child.startswith('.'), size, modified_date, created_date)
        tree_node.add_children(file_system_child_node)
        tree_node.has_children = True
        return file_system_child_node

    def get_full_path(self, *children):
        return join(self.root_folder, *children)

    def generate_files_system_view(self, show_hidden_files, files_type, name_filter, sorting_criteria, reverse_order):
        files_system_view = FileSystemView(self.root_tree_node, show_hidden_files, files_type, name_filter, sorting_criteria, reverse_order )
        return files_system_view


class Controller(object):

    def __init__(self, files_system_view):
        self.files_system_view = files_system_view
        self.root_tree_node_view = self.files_system_view.get_file_system_tree_node()
        self.actions = []

    def populate_actions(self, action_button_group):
        (action_descriptor, action_args) = actiongroup.get_inputs()
        action_class = action_descriptor.action_class
        action_instance = action_class(path_part, **action_args)
        self.actions.append(action_instance)


    def execute_method_on_nodes(self, tree_node, method, *optional_argument):
        """Execute a method on a given FileSystemTreeNode with zero or more optional arguments."""
        if tree_node.get_original_relative_path() != self.root_tree_node_view.get_original_relative_path():
            #Do not apply the actions to the selected directory.
            method(tree_node, *optional_argument)
        for child in tree_node.get_children():
            self.execute_method_on_nodes(child, method, *optional_argument)



    def batch_update(self, actions, file_or_folder):
        self.execute_method_on_nodes(self.root_tree_node_view, self.update, actions, file_or_folder)

    def call_actions(self, tree_node, actions, file_or_folder):
        for action in actions:
            tree_node = action.call(tree_node, file_or_folder)

    def update(self, tree_node, actions, file_or_folder):
        self.reset(tree_node)
        self.call_actions(tree_node, actions, file_or_folder)

    def reset(self, tree_node):
        """Reset the modified filedescriptor with the original one."""
        tree_node.modified_filedescriptor = deepcopy(tree_node.original_filedescriptor)
        return tree_node

    def rename(self, tree_node):
        try:
            self.has_duplicates(tree_node)
            if exists(tree_node.get_modified_path()):
                conflicting_tree_node = tree_node.get_parent().find_child_by_path(tree_node.get_modified_path())
                if(conflicting_tree_node is not None and tree_node.get_file_system_tree_node() != conflicting_tree_node): #check if it is the same file
                    old_conflicting_file_descriptor = conflicting_tree_node.modified_filedescriptor
                    conflicting_tree_node.modified_filedescriptor = FileDescriptor(str(uuid4()) + "."+ tree_node.original_filedescriptor.extension, tree_node.is_folder)
                    self.rename(conflicting_tree_node)
                    conflicting_tree_node.modified_filedescriptor = old_conflicting_file_descriptor
            move(tree_node.get_original_path(), tree_node.get_modified_path())
            tree_node.original_filedescriptor = deepcopy(tree_node.modified_filedescriptor)
            # self.reset(tree_node)
        except IOError as e:
            raise Exception(str(e))

    def undo(self, tree_node):
        move(tree_node.get_original_path(), tree_node.get_backup_path())
        tree_node.original_filedescriptor = tree_node.backup_filedescriptor

    def has_duplicates(self, tree_node):
        """Finds if there are duplicate files/folders. If there are some duplicates, appends a counter to differenciate them."""
        children_names = []
        if tree_node.get_parent() is not None:
            for same_level_tree_node in tree_node.get_parent().get_children():
                if (same_level_tree_node.modified_filedescriptor.basename in children_names):
                    raise Exception("Names conflict: several items have the same name. Please choose new options to avoid duplicates.") 
                else:
                    children_names.append(same_level_tree_node.modified_filedescriptor.basename)


    def batch_rename(self):
        self.execute_method_on_nodes(self.root_tree_node_view, self.rename)

    def batch_undo(self):
        self.execute_method_on_nodes(self.root_tree_node_view, self.undo)

    def convert_tree_to_list(self):
        flat_tree_list = []
        self.execute_method_on_nodes(self.root_tree_node_view, flat_tree_list.append)
        for tree_node in flat_tree_list:
            self.flat_tree_list.append(tree_node.get_modified_path())
        return self.flat_tree_list

    def save_result_to_file(self, name, input_list):
        directory = join(dirname(__file__),"UnitTest", "TestCase1_Models", name)
        file = open(directory, 'w+')
        for line in input_list:
            file.writelines(line + '\n')
        file.close()

    def parse_renamed_files(self, input_path, sorting_criteria, reverse_order):
        input_path = abspath(input_path)
        directory_files = sorted(listdir(input_path), key = lambda file : self.get_file_sorting_criteria(join(input_path, file), sorting_criteria), reverse=reverse_order)
        for filename in directory_files:
            filepath = join(input_path, filename)
            if filename[0] == '.' and not self.show_hidden_files:
                continue
            if isdir(filepath):
                self.renamed_files_list.append(relpath(filepath, self.root_folder))
                if (not self.use_subdirectory):
                    continue
                else:
                    self.parse_renamed_files(filepath, sorting_criteria, reverse_order)
            else:
                self.renamed_files_list.append(relpath(filepath, self.root_folder))

    def get_renamed_files(self):
        self.save_result_to_file("hooh", self.renamed_files_list)




#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import pyqtSignal, QRect, Qt, QSize
from PyQt5.QtWidgets import QWidget, QFrame, QGridLayout, QLabel, QPushButton, QComboBox, QCheckBox, QLineEdit, QSpinBox, QRadioButton
from PyQt5.QtGui import QFont, QIcon
from ..model import action_manager
from . import resource_rc


class ActionButton(QWidget):
    action_changed = pyqtSignal(object, object)  # get changes in order to refresh the preview.

    def __init__(self, action_descriptors, are_sub_buttons):
        QWidget.__init__(self)
        self.grid = QGridLayout(self)
        self.are_sub_buttons = are_sub_buttons
        self.button_inputs_dict = {}
        self.combobox = QComboBox()
        self.combobox.setObjectName("action_selector")
        self.action_descriptors = action_descriptors
        for element in self.action_descriptors:
            self.combobox.addItem(str(element))
        self.grid.addWidget(self.combobox, 0, 0, 1, 1)
        self.combobox.currentIndexChanged[int].connect(self.on_combobox_changed)
        self.grid.setRowStretch(2,1)
        self.populate_widget(self.action_descriptors[0])
        self.store_inputs()

    def change(self):
        ''' Change occurs on the layout. '''
        self.store_inputs()
        self.action_changed.emit(self.selected_action, self.action_inputs)

    def on_combobox_changed(self, index):
        self.button_inputs_dict = {}
        self.populate_widget(self.action_descriptors[index])
        self.change()

    def populate_widget(self, current_combo_selection):
        sub_buttons = self.grid.itemAtPosition(1,0)
        if sub_buttons is not None:
            widgetToRemove = sub_buttons.widget()
            # get it out of the layout list
            self.grid.removeWidget(widgetToRemove)
            # remove it form the gui
            widgetToRemove.setParent(None)

        if isinstance(current_combo_selection, action_manager.ActionDescriptorGroup):
            subframe = QFrame(self)
            subframe.setObjectName("subframe")
            subframe.setStyleSheet("QFrame#subframe{border:2px solid rgb(220, 220, 220); border-radius:3px; padding:0px; background-color: rgb(253, 253, 253)};")
            sub_grid = QGridLayout(subframe)
            sub_grid.setObjectName("subgrid")
            font = QFont()
            font.setWeight(55)
            font.setBold(True)
            option_label = QLabel("options :")
            option_label.setFont(font)
            sub_grid.addWidget(option_label, 0, 0, 1, 1)
            self.sub_action_group = ActionButton(current_combo_selection.action_descriptors, True)
            self.sub_action_group.action_changed.connect(self.change)
            sub_grid.addWidget(self.sub_action_group, 1, 0, 1, 1)
            self.grid.addWidget(subframe,1,0,1,1)
            return
        if current_combo_selection.action_inputs != []:
            subframe = QFrame(self)
            subframe.setObjectName("subframe")
            sub_grid = QGridLayout(subframe)
            sub_grid.setObjectName("subgrid")
            if self.are_sub_buttons is False:
                subframe.setStyleSheet("QFrame#subframe{border:2px solid rgb(220, 220, 220); border-radius:3px; padding:0px; background-color: rgb(253, 253, 253)};")
                font = QFont()
                font.setWeight(55)
                font.setBold(True)
                option_label = QLabel("options :")
                option_label.setFont(font)
                sub_grid.addWidget(option_label, 0, 0, 1, 1)
            for i, arguments in enumerate(current_combo_selection.action_inputs):
                label = QLabel()
                label.setObjectName("label")
                label.setText(str(arguments.argument_caption))
                if arguments.argument_type == str:
                    sub_button = QLineEdit()
                    sub_button.setText(arguments.default_value)
                    sub_button.textChanged[str].connect(self.get_text_changed)
                elif arguments.argument_type == "checkable":
                    sub_button = QCheckBox()
                    sub_button.setChecked(arguments.default_value)
                    sub_button.stateChanged[int].connect(self.get_state_changed)
                elif arguments.argument_type == bool:
                    sub_button = QRadioButton()
                    sub_button.setText(str(arguments.argument_caption))
                    label.setText("")
                    sub_button.setChecked(arguments.default_value)
                    sub_button.toggled.connect(self.radio_button_clicked)
                elif arguments.argument_type == int:
                    sub_button = QSpinBox()
                    sub_button.setValue(arguments.default_value)
                    sub_button.valueChanged[int].connect(self.get_integer_changed)
                sub_button.setObjectName(str(arguments.argument_name))
                if(label.text() == ""):
                    sub_grid.addWidget(sub_button, i+1, 0, 1, 2)
                else:
                    sub_grid.addWidget(label, i+1, 0, 1, 1)
                    sub_grid.addWidget(sub_button, i+1, 1, 1, 1)
                self.button_inputs_dict[arguments.argument_name] = arguments.default_value
            self.grid.addWidget(subframe,1,0,1,1)

    def radio_button_clicked(self, enabled):
        if enabled:
            self.button_inputs_dict[self.sender().objectName()] = True
            self.change()
        else:
            self.button_inputs_dict[self.sender().objectName()] = False

    def get_text_changed(self, value):
        self.button_inputs_dict[self.sender().objectName()] = value
        self.change()

    def get_state_changed(self, value):
        state = None
        if value == 0:
            state = False
        elif value == 2:
            state = True
        self.button_inputs_dict[self.sender().objectName()] = state
        self.change()

    def get_integer_changed(self, value):
        if value=="":
            value = 0
        self.button_inputs_dict[self.sender().objectName()] = value
        self.change()

    def get_inputs(self):
        return self.selected_action, self.action_inputs

    def store_inputs(self):
        if isinstance(self.action_descriptors[self.combobox.currentIndex()], action_manager.ActionDescriptorGroup):
            (self.selected_action, self.action_inputs) = self.sub_action_group.get_inputs()
        else:
            self.selected_action = self.action_descriptors[self.combobox.currentIndex()]
            self.action_inputs = self.button_inputs_dict

class ActionButtonGroup(QWidget):
    changed = pyqtSignal() # get changes in order to refresh the preview.
    """Group the combobox with the textboxes containing the subactions"""
    removed = pyqtSignal(QWidget) # emit a signal when the widget is removed.
    addedBefore = pyqtSignal(QWidget) 
    addedAfter = pyqtSignal(QWidget)
    button_stylesheet = """
    QPushButton {
         border: none;
         border-radius: 6px;
         min-width: 25px;
     }

    QPushButton:hover {
        border: 1px solid rgb(180,180,180);
        background-color: rgb(210, 210, 210);
        border-radius: 6px;
    }
    """

    def __init__(self, frame_name, action_descriptors, frame_width, frame_height, frame_type):
        QWidget.__init__(self)
        self.maximum_height_size = frame_height
        self.maximum_width_size = frame_width
        self.frame_type = frame_type
        self.frame = QFrame(self)
        self.frame.setObjectName("frame")
        if ("fix" in frame_type):
            self.frame.setStyleSheet("QFrame#frame{border:2px solid rgb(210, 210, 210); border-radius: 7px; padding:0px; background-color: rgb(230, 230, 230)};")
        else:
            self.frame.setStyleSheet("QFrame#frame{border:2px solid rgb(220, 220, 220); border-radius: 7px; padding:0px; background-color: rgb(244, 244, 244)};")
        self.frame_grid = QGridLayout(self.frame) #this is a hidden grid to handle the objects in the frame as if it was a grid.
        self.frame_grid.setObjectName("frame_grid")
        self.frame.setGeometry(QRect(0, 0, self.maximum_width_size, self.maximum_height_size))
        self.frame_name = frame_name
        self.label = QLabel(self.frame_name)
        self.label.setAlignment(Qt.AlignCenter)
        font = QFont()
        font.setWeight(75)
        font.setBold(True)
        self.label.setFont(font)
        remove_widget = QPushButton()
        remove_widget.setFixedSize(25,25)
        remove_widget.setIcon(QIcon(":/delete_icon.png"))
        remove_widget.setIconSize(QSize(16,16))
        remove_widget.setFlat(True)
        remove_widget.setStyleSheet(self.button_stylesheet)
        remove_widget.pressed.connect(self.on_remove_widget)
        add_prefix = QPushButton()
        add_prefix.setFixedSize(25,25)
        add_prefix.setIcon(QIcon(":/add_icon.png"))
        add_prefix.setFlat(True)
        add_prefix.setStyleSheet(self.button_stylesheet)
        add_prefix.setIconSize(QSize(16,16))
        add_prefix.pressed.connect(self.on_add_prefix)
        add_suffix = QPushButton()
        add_suffix.setFixedSize(25,25)
        add_suffix.setIcon(QIcon(":/add_icon.png"))
        add_suffix.setIconSize(QSize(16,16))
        add_suffix.setFlat(True)
        add_suffix.setStyleSheet(self.button_stylesheet)
        add_suffix.pressed.connect(self.on_add_suffix)
        self.grid = QGridLayout()
        self.grid.setObjectName("grid")
 
        self.grid.addWidget(self.label, 0, 1, 1, 1)
        if (self.frame_type == "prefix"):
            self.grid.addWidget(add_prefix, 0, 0 ,1 ,1)
            self.grid.addWidget(remove_widget, 0 ,2 ,1 ,1)
        if (self.frame_type == "suffix"):
            self.grid.addWidget(add_suffix, 0 ,2 ,1 ,1)
            self.grid.addWidget(remove_widget, 0 ,0 ,1 ,1)
        if (self.frame_type == "file" or frame_type == "folder"):
            self.grid.addWidget(add_prefix, 0 ,0 ,1 ,1)
            self.grid.addWidget(add_suffix, 0 ,2 ,1 ,1)
        if (self.frame_type =="extension"):
            empty = QWidget();
            empty.setMinimumSize(add_suffix.size())
            self.grid.addWidget(empty, 0 ,0 ,1 ,1)
            self.grid.addWidget(empty, 0 ,2 ,1 ,1)
        self.action_group = ActionButton(action_descriptors, False)
        self.action_group.action_changed.connect(self.change)
        self.grid.addWidget(self.action_group, 1, 0, 1, 3)
        self.frame_grid.addLayout(self.grid, 0, 0, 1, 1)

    def get_action_with_inputs(self):
        (selected_action_descriptor, action_args) = self.action_group.get_inputs()
        action_class = selected_action_descriptor.action_class
        action_instance = action_class(self.frame_type, **action_args)
        return action_instance


    def change(self, selected_action, button_inputs):
        '''Emit signal when added.'''
        self.selected_action = selected_action
        self.action_inputs = button_inputs
        self.changed.emit()

    def on_add_prefix(self):
        '''Emit signal when added.'''
        self.addedBefore.emit(self)

    def on_add_suffix(self):
        '''Emit signal when added.'''
        self.addedAfter.emit(self)

    def on_remove_widget(self):
        '''Emit signal when removed.'''
        self.removed.emit(self)

    def set_label(self, new_label):
        self.frame_name = new_label
        self.label.setText(new_label)

    def get_action_descriptors(self):
        return self.action_descriptors

    def get_frame_type(self):
        return self.frame_type

    def get_frame_name(self):
        return self.frame_name

    def set_frame_type(self, value):
        self.frame_type = value


    def get_maximum_height(self):
        if (self.maximum_height_size < self.frame.minimumSizeHint().height()):
            self.maximum_height_size = self.frame.minimumSizeHint().height()
        return self.maximum_height_size

    def get_maximum_width(self):
        if (self.maximum_width_size < self.frame.minimumSizeHint().width()):
            self.maximum_width_size = self.frame.minimumSizeHint().width()
        return self.maximum_width_size


    def clearLayout(self, layout):
        """delete all children of the specified layout"""
        for i in reversed(range(layout.count())): 
                        widgetToRemove = layout.itemAt(i).widget()
                        # get it out of the layout list
                        layout.removeWidget( widgetToRemove )
                        # remove it form the gui
                        widgetToRemove.setParent( None )

    def destruct_layout(self):
        """Delete entire layout."""
        while self.grid.count():
            child = self.grid.takeAt(0)
            if child.widget() is not None:
                child.widget().deleteLater()
            elif child.layout() is not None:
                self.clearLayout(child.layout())
        self.deleteLater()

    def on_show_information(self, warning_message):
        """Show the information message"""
        QMessageBox.information(self, "Information", warning_message)




#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
import re 


class FileSystemTreeNodeView(object):
    """
    Represents a portion of the FileSystemTreeNode depending on filters chosen by the users.
    Parameters:
        --file_system_tree_node: original filesystemtreenode
        --rank: integer that represents the position of the current file/folder in the list of FileSystemTreeNode children.
    """
    def __init__(self, file_system_tree_node, rank):
        self._rank = rank
        self.children = []
        self.files_system_tree_node = file_system_tree_node

    def __repr__(self):
        return self.files_system_tree_node.__repr__()

    def get_file_system_tree_node(self):
        return self.files_system_tree_node

    def add_children(self, file_system_tree_node_view):
        self.children.append(file_system_tree_node_view)

    def get_children(self):
        return self.children

    def find_child_by_path(self, path):
        return self.files_system_tree_node.find_child_by_path(path)

    def get_parent(self):
        return self.files_system_tree_node.get_parent()

    def get_original_relative_path(self):
        return self.files_system_tree_node.get_original_relative_path()

    def get_modified_relative_path(self):
        return self.files_system_tree_node.get_modified_relative_path()

    def get_backup_relative_path(self):
        return self.files_system_tree_node.get_backup_relative_path()

    def get_original_path(self):
        return self.files_system_tree_node.get_original_path()

    def get_modified_path(self):
        return self.files_system_tree_node.get_modified_path()

    def get_backup_path(self):
        return self.files_system_tree_node.get_backup_path()
    @property
    def size(self):
        return self.files_system_tree_node.size
    @property
    def modified_date(self):
        return self.files_system_tree_node.modified_date
    @property
    def created_date(self):
        return self.files_system_tree_node.created_date

    @property
    def is_folder(self):
        return self.files_system_tree_node.is_folder

    @property
    def original_filedescriptor(self):
        return self.files_system_tree_node.original_filedescriptor

    @original_filedescriptor.setter
    def original_filedescriptor(self, value):
        self.files_system_tree_node.original_filedescriptor = value

    @property
    def modified_filedescriptor(self):
        return self.files_system_tree_node.modified_filedescriptor

    @modified_filedescriptor.setter
    def modified_filedescriptor(self, value):
        self.files_system_tree_node.modified_filedescriptor = value

    @property
    def backup_filedescriptor(self):
        return self.files_system_tree_node.backup_filedescriptor

    @backup_filedescriptor.setter
    def backup_filedescriptor(self, value):
        self.files_system_tree_node.backup_filedescriptor = value

    @property
    def rank(self):
        """Give the position of the file according to the sorting criteria."""
        return self._rank

    @rank.setter
    def rank(self, value):
        self._rank = value
        self.files_system_tree_node.rank = value

    def match_files_type(self, files_type):
        self.files_system_tree_node.match_files_type(files_type)

    def match_name_filter(self, name_filter):
        self.files_system_tree_node.match_name_filter(name_filter)

    @property
    def is_hidden(self):
        self.files_system_tree_node.is_hidden

    @property
    def has_children(self):
        return self.files_system_tree_node.has_children


class FileSystemView(object):
    """
    Represents a portion of the FilesSystem. This portion depends on the following filters:
    Parameters:
        --show_hidden_files: whether or not to show the hidden files.
    """

    def __init__(self, root_tree_node, show_hidden_files, files_type, name_filter, sorting_criteria, reverse_order):
        self.show_hidden_files = show_hidden_files
        self.files_type = files_type
        self.name_filter = name_filter
        self.sorting_criteria = sorting_criteria
        self.reverse_order = reverse_order
        self.root_tree_node_view = FileSystemTreeNodeView(root_tree_node, 0)
        self.filter_files(root_tree_node, self.root_tree_node_view)

    def filter_files(self, tree_node, tree_node_view):
        rank_file = -1
        rank_folder = -1
        for tree_node_child in sorted(tree_node.get_children(), key = lambda node: self.get_sorting_key(node), reverse=self.reverse_order):
            if (self.is_filtered_tree_node(tree_node_child) is True):
                continue
            if (tree_node_child.is_folder):
                rank_folder += 1
                rank = rank_folder
            else:
                rank_file +=1
                rank = rank_file
            tree_node_child_view = FileSystemTreeNodeView(tree_node_child, rank)
            tree_node_view.add_children(tree_node_child_view)
            if (tree_node_child.is_folder):
                self.filter_files(tree_node_child, tree_node_child_view)

    def is_filtered_tree_node(self, tree_node):
        if (not self.show_hidden_files and tree_node.is_hidden):
            return True
        if(tree_node.has_children is False and tree_node.is_folder is True and self.files_type != ["folders"]):
            return True
        if (tree_node.is_folder and tree_node.has_children):
            return False
        if (not tree_node.match_files_type(self.files_type)):
            return True
        if (not tree_node.match_name_filter(self.name_filter)):
            return True
        return False

    def get_file_system_tree_node(self):
        return self.root_tree_node_view
 
    def natural_sort(self, tree_node):
        """ Sorts the given iterable in the way that is expected.
        """
        filename = tree_node.original_filedescriptor.basename
        convert = lambda text: int(text) if text.isdigit() else text
        alphanum_key = [convert(c) for c in re.split('([0-9]+)', filename)]
        return alphanum_key

    def get_sorting_key(self, tree_node):
        """
        Criteria to sort the files.
        Parameters:
            --tree_node: path to the specified file/folder.
            --sorting_criteria: string that specifies the sorting criteria. Default is 'name'. Possible values are : name, size, creation_date and modified_date.
        """
        if self.sorting_criteria == "size":
            return tree_node.size
        elif self.sorting_criteria == "modified_date":
            return tree_node.modified_date
        elif self.sorting_criteria ==  "creation_date":
            return tree_node.created_date
        elif self.sorting_criteria == "name":
            return self.natural_sort(tree_node)
        else:
            return None



#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
from time import strftime, strptime, localtime
from re import sub
from exifread import process_file
from mutagen.easyid3 import EasyID3

class ActionDescriptorGroup(object):
    """
    Contains a collectino of ActionDescriptor.
    Parameters:
        --action_descriptors: a list of ActionDescriptor.
    """
    def __init__(self, action_name, action_descriptors, action_class):
        self.action_name = action_name
        self.action_descriptors = action_descriptors
        self.action_class = action_class

    def __repr__(self):
        """override string representation of the class"""
        return self.action_name

class ActionDescriptor(object):
    """
    Describes the actions by names, inputs and classes.
    Parameters:
        --action_name: string that represents the name of the action.
        --action_inputs: list of ActionInput that represents the inputs properties of the acion.
        --action_class: string that represents the name of the class used for the action.
    """
    def __init__(self, action_name, action_inputs, action_class):
        self.action_name = action_name
        self.action_inputs = action_inputs
        self.action_class = action_class

    def __repr__(self):
        """override string representation of the class"""
        return self.action_name

class ActionInput(object):
    """
    Describes the inputs properties of the action.
    Parameters:
        --arg_name: string that represents the name of the given parameter.
        --arg_caption: string that represents the caption of the given parameter.
        --arg_type: specifies which type is the given parameter.
        --default_value: specifies the default value of the given parameter.
        --optional_argument: gives the possibility to add an optional argument for storing data.
    """
    def __init__(self, arg_name, arg_caption, arg_type, default_value, optional_argument = None):
        self.argument_name = arg_name
        self.argument_caption = arg_caption
        self.argument_type = arg_type
        self.default_value = default_value
        self.optional_argument = optional_argument

class Action(object):
    """
    Describes how the action is applied on the FileSystemTreeNodes. This class is inherited by all the specific actions.
    Parameters:
        --path_type: string that represents where the action will be applied. path_type can be 'folder', 'file', 'prefix', 'suffix' or 'extension'.
        --file_or_folder: specifies where to apply the actions : to the files or the folders.
    """
    def __init__(self, path_type):
        self.path_type = path_type

    def call(self, file_system_tree_node, file_or_folder):
        """Apply action on the specified part."""
        prefix = ""
        suffix = ""
        # pdb.set_trace()
        if (file_or_folder == "folder"):
            if (file_system_tree_node.is_folder):
                if(self.path_type == "folder"):
                    file_system_tree_node.modified_filedescriptor.foldername = self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.foldername)
                elif(self.path_type == "suffix"):
                    file_system_tree_node.modified_filedescriptor.suffix = file_system_tree_node.modified_filedescriptor.suffix + self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.suffix)
                elif(self.path_type == "prefix"):
                    file_system_tree_node.modified_filedescriptor.prefix = file_system_tree_node.modified_filedescriptor.prefix + self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.prefix)
                elif(self.path_type == "extension"):
                    return file_system_tree_node
                else:
                    raise Exception("path_type not valid")

        elif (file_or_folder == "file"):
            if (file_system_tree_node.is_folder is False):
                if(self.path_type == "file"):
                    file_system_tree_node.modified_filedescriptor.filename = self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.filename)
                elif(self.path_type == "suffix"):
                    file_system_tree_node.modified_filedescriptor.suffix = file_system_tree_node.modified_filedescriptor.suffix + self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.suffix)
                elif(self.path_type == "prefix"):
                    file_system_tree_node.modified_filedescriptor.prefix = file_system_tree_node.modified_filedescriptor.prefix + self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.prefix)
                elif(self.path_type == "extension"):
                    file_system_tree_node.modified_filedescriptor.extension = self.call_on_path_part(file_system_tree_node, file_system_tree_node.modified_filedescriptor.extension)
                else:
                    raise Exception("path_type not valid")
        return file_system_tree_node

    def call_on_path_part(self, file_system_tree_node, path_part):
        raise Exception("not implemented")



class CharacterReplacementAction(Action):
    """
    Replace old_char by new_char in the section of the path.
    path_part can be 'folder', 'file', 'prefix', 'suffix' or 'extension'.
    """
    def __init__(self, path_type, old_char, new_char, regex):
        Action.__init__(self, path_type)
        self.old_char = old_char
        self.new_char = new_char
        self.regex = regex

    def call_on_path_part(self, file_system_tree_node, path_part):
        if not self.regex:
            return path_part.replace(self.old_char,self.new_char)
        else:
            return sub(self.old_char, self.new_char, path_part)

class OriginalNameAction(Action):
    """Gets the original name."""
    def __init__(self, path_type):
        Action.__init__(self, path_type)

    def call_on_path_part(self, file_system_tree_node, path_part):
        return path_part

class CaseChangeAction(Action):
    """
    Return the original name with a chosen casing option.
    Parameters:
        --case_choise: option to specify the case : 'uppercase', 'lowercase', 'titlecase'.
        --first_letter: boolean making first letter uppercase or lowercase.
        --after_symbols: list of symbols after which the letters are capitalized.
    """
    def __init__(self, path_type):
        Action.__init__(self, path_type)

    def call_on_path_part(self, file_system_tree_node, path_part):
        return path_part

class TitleCaseAction(CaseChangeAction):
    """
    Return the original name with a chosen casing option.
    Parameters:
        --case_choise: option to specify the case : 'uppercase', 'lowercase', 'titlecase'.
        --first_letter: boolean making first letter uppercase or lowercase.
        --after_symbols: list of symbols after which the letters are capitalized.
    """
    def __init__(self, path_type, first_letter = True, after_symbols = ""):
        CaseChangeAction.__init__(self, path_type)
        self.first_letter = first_letter
        self.after_symbols = after_symbols

    def make_upper_first_letters(self, string):
        special_char_position = []
        stringlist = list(string)
        for i, char in enumerate(stringlist):
            if char in self.after_symbols:
                special_char_position.append(i+1)
        for position in special_char_position:
            if position < len(stringlist):
                stringlist[position] = stringlist[position].upper()
        return ''.join(stringlist)

    def call_on_path_part(self, file_system_tree_node, path_part):
       path_part = self.make_upper_first_letters(path_part)
       if self.first_letter:
           path_part = path_part[0].upper() + path_part[1:]
       return path_part

class UpperCaseAction(CaseChangeAction):
    """
    Return the original name with a chosen casing option.
    Parameters:
        --case_choise: option to specify the case : 'uppercase', 'lowercase', 'titlecase'.
        --first_letter: boolean making first letter uppercase or lowercase.
        --after_symbols: list of symbols after which the letters are capitalized.
    """
    def __init__(self, path_type):
        CaseChangeAction.__init__(self, path_type)

    def call_on_path_part(self, file_system_tree_node, path_part):
        return path_part.upper()

class LowerCaseAction(CaseChangeAction):
    """
    Return the original name with a chosen casing option.
    Parameters:
        --case_choise: option to specify the case : 'uppercase', 'lowercase', 'titlecase'.
        --first_letter: boolean making first letter uppercase or lowercase.
        --after_symbols: list of symbols after which the letters are capitalized.
    """
    def __init__(self, path_type):
        CaseChangeAction.__init__(self, path_type)

    def call_on_path_part(self, file_system_tree_node, path_part):
        return path_part.lower()

class CharacterInsertionAction(Action):
    """Insert new_char at index position."""
    def __init__(self, path_type, new_char, index):
        Action.__init__(self, path_type)
        self.new_char = new_char
        self.index = index

    def call_on_path_part(self, file_system_tree_node, path_part):
        return path_part[:self.index] + self.new_char + path_part[self.index:]

class CharacterDeletionAction(Action):
    """Delete n-character from starting_position to ending_position."""
    def __init__(self, path_type, starting_position, ending_position):
        Action.__init__(self, path_type)
        self.starting_position = starting_position
        self.ending_position = ending_position

    def call_on_path_part(self, file_system_tree_node, path_part):
        if self.starting_position > self.ending_position:
            raise Exception("Starting position cannot be higher than ending position.")
        return path_part[:self.starting_position] + path_part[self.ending_position:]

class CustomNameAction(Action):
    """Use a custom name in the filename."""
    def __init__(self, path_type, new_name):
        Action.__init__(self, path_type)
        self.new_name = new_name

    def call_on_path_part(self, file_system_tree_node, path_part):
        return self.new_name

class FolderNameUsageAction(Action):
    """Use the parent foldername as the filename."""
    def __init__(self, path_type):
        Action.__init__(self, path_type)

    def call_on_path_part(self, file_system_tree_node, path_part):
        folder = file_system_tree_node.get_parent().original_filedescriptor.basename
        return folder

class DateAction(Action):
    """
    Use the created or modified date metadata as the filename.
    If is_modified_time = True, the modified date from the file metadata is taken. Otherwise, it is the created date.
    Commonly used format_display are :
    %Y  Year with century as a decimal number.
    %m  Month as a decimal number [01,12].
    %d  Day of the month as a decimal number [01,31].
    %H  Hour (24-hour clock) as a decimal number [00,23].
    %M  Minute as a decimal number [00,59].
    %S  Second as a decimal number [00,61].
    %z  Time zone offset from UTC.
    %a  Locale's abbreviated weekday name.
    %A  Locale's full weekday name.
    %b  Locale's abbreviated month name.
    %B  Locale's full month name.
    %c  Locale's appropriate date and time representation.
    %I  Hour (12-hour clock) as a decimal number [01,12].
    %p  Locale's equivalent of either AM or PM.
    """
    def __init__(self, path_type, is_modified_date = False, is_created_date = True, format_display = '%Y'):
        Action.__init__(self, path_type)
        self.is_modified_date = is_modified_date
        self.is_created_date = is_created_date
        self.format_display = format_display

    def call_on_path_part(self, file_system_tree_node, path_part):
        if self.is_modified_date:
            file_date = file_system_tree_node.modified_date
        elif self.is_created_date:
            file_date = file_system_tree_node.created_date
        return strftime(self.format_display, localtime(file_date))

class Counter(Action):
    """Count the number of files starting from start_index with the given increment."""
    def __init__(self, path_type, start_index, increment, digit_number):
        Action.__init__(self, path_type)
        self.start_index = start_index
        self.increment = increment
        self.digit_number = digit_number

    def call_on_path_part(self, file_system_tree_node, path_part):
        counter = file_system_tree_node.rank
        counter *= self.increment
        counter += self.start_index
        counter = str(counter)
        number_length = len(str(counter))
        if (number_length < self.digit_number):
            for i in range(self.digit_number - number_length):
                counter = "0" + counter
        return counter

class GenericImageAction(Action):
    def __init__(self, path_type, metadata):
        Action.__init__(self, path_type)
        self.metadata = metadata

    def get_exif_tag(self, file):
        with open(file, 'rb') as f:
            tags = process_file(f, details=False, stop_tag=self.metadata)
            exif_tag = tags[self.metadata].values
            return exif_tag

class ImageDateTimeOriginal(GenericImageAction):
    def __init__(self, path_type, time_format):#, time_format):
        #     self.time_format = time_format
        GenericImageAction.__init__(self, path_type, 'EXIF DateTimeOriginal')
        self.time_format = time_format

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            localtime = strptime(exif_tag, "%Y:%m:%d %H:%M:%S")
            return strftime(self.time_format, localtime)
        except:
            return path_part


class ImageFNumber(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF FNumber')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0].num/exif_tag[0].den)
        except:
            return path_part        

class ImageExposureTime(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF ExposureTime')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0].num/exif_tag[0].den)
        except:
            return path_part     

class ImageISO(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF ISOSpeedRatings')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0])
        except:
            return path_part     

class ImageCameraModel(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'Image Model')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return exif_tag
        except:
            return path_part  

class ImageXDimension(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF ExifImageWidth')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0])
        except:
            return path_part  

class ImageYDimension(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF ExifImageLength')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0])
        except:
            return path_part  

class ImageFocalLength(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'EXIF FocalLength')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return str(exif_tag[0].num/exif_tag[0].den)
        except:
            return path_part  

class ImageArtist(GenericImageAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'Image Artist')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            exif_tag = self.get_exif_tag(file_system_tree_node.get_original_path())
            return exif_tag
        except:
            return path_part  



class GenericMusicAction(Action):
    def __init__(self, path_type, metadata):
        Action.__init__(self, path_type)
        self.metadata = metadata

    def get_metadata_tag(self, file):
        audio = EasyID3(file)
        return ', '.join(audio[self.metadata])


class MusicArtist(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'artist')

    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part

class MusicTitle(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'title')
    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part

class MusicYear(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'date')
    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part

class MusicAlbum(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'album')
    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part

class MusicTrack(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'tracknumber')
    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part

class MusicGenre(GenericMusicAction):
    def __init__(self, path_type):
        GenericImageAction.__init__(self, path_type, 'genre')
    def call_on_path_part(self, file_system_tree_node, path_part):
        try:
            metadata_tag = self.get_metadata_tag(file_system_tree_node.get_original_path())
            return metadata_tag
        except:
            return path_part




import sys
from setuptools import setup
import whiterenamer

if sys.version_info.major < 3:
    print('This package is compatible with Python 3 and above.')
    exit(4)

setup(
    name = "whiterenamer",
    version = whiterenamer.__version__,
    description = "User friendly batch renamer",
    author = "Pierre Blanc",
    author_email = "pierrecnalb@mailbox.org",
    url = 'https://github.com/pierrecnalb/White-Renamer',
    license='GNU Public General License',
    packages = ['whiterenamer', 'whiterenamer.ui', 'whiterenamer.model', 'whiterenamer.doc'],
    zip_safe = False,
    entry_points={ 'console_scripts': [ 'whiterenamer = whiterenamer:start', ] },
    include_package_data = True,
    test_suite="tests.whiterenamer_suite",
    tests_require="nose",
    keywords='batch renamer gui simple exif photo music metadata',
    install_requires=['mutagen', 'exifread'],
)



#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.
from . import run, model, ui

class Version(object):
    def __init__(self, major, minor, revision, maturityLevel='', maturationStep=None):
        if maturityLevel not in ('a', 'b', ''):
            raise Exception('Unknown maturity level.')
        self.major = major
        self.minor = minor
        self.revision = revision
        self.maturityLevel = maturityLevel
        self.maturationStep = maturationStep

        if self.isRelease != (self.maturationStep == None):
            raise Exception('A maturation step is expected if and only if not a release version.')

    @property
    def isRelease(self):
        return self.maturityLevel == ''

    @property
    def components(self):
        if self.maturityLevel == '':
            return [self.major, self.minor, self.revision]
        elif self.maturationStep == None:
            return [self.major, self.minor, self.revision, self.maturityLevel]
        else:
            return [self.major, self.minor, self.revision, self.maturityLevel, self.maturationStep]

    def __eq__(self, other):
        return self._compareWith(other, 0)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __gt__(self, other):
        return self._compareWith(other, 1)

    def __ge__(self, other):
        return self.__gt__(other) or self.__eq__(other)

    def __lt__(self, other):
        return self._compareWith(other, -1)

    def __le__(self, other):
        return self.__lt__(other) or self.__eq__(other)

    def _compareWith(self, other, comparisonType):
        if other == None:
            return False

        selfComponents = self.components
        otherComponents = other.components

        if comparisonType == 0: # Equality.
            return selfComponents == otherComponents
        else:
            for i in range(min(len(selfComponents), len(otherComponents))):
                c1 = selfComponents[i]
                c2 = otherComponents[i]

                # Replace the empty maturity level with a 'r' that naturally
                # compares greater than 'a' or 'b'.
                if i == 3:
                    c1 = 'r' if c1 == '' else c1
                    c2 = 'r' if c2 == '' else c2
                if comparisonType == -1: # Lower than.
                    if c1 < c2: return True
                    elif c1 > c2: return False
                elif comparisonType == 1: # Greater than.
                    if c1 > c2: return True
                    elif c1 < c2: return False
                else:
                    raise Exception('Invalid comparison type.')

            # If we do not have the same component count on both sides, the
            # longest is assumed to be older than the other (1.2.3b1 is older
            # than 1.2.3)
            if comparisonType == -1:
                return len(selfComponents) > len(otherComponents)
            elif comparisonType == 1:
                return len(selfComponents) < len(otherComponents)

    def __repr__(self):
        if self.maturityLevel == '':
            return '{major}.{minor}.{revision}'.format(**vars(self))
        else:
            if self.maturationStep == None:
                return '{major}.{minor}.{revision}{maturityLevel}'.format(**vars(self))
            else:
                return '{major}.{minor}.{revision}{maturityLevel}{maturationStep}'.format(**vars(self))

def start():
    run.main()

version = Version(1, 0, 0)
__version__=str(version)
print(__version__)




#!/usr/bin/python3

# Copyright (C) 2015-2016 Pierre Blanc
#
# This file is part of WhiteRenamer.
#
# WhiteRenamer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# WhiteRenamer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with WhiteRenamer. If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtWidgets import QWidget, QGridLayout, QTreeView, QSizePolicy, QScrollArea, QHBoxLayout, QFrame, QLabel, QMessageBox, QAbstractItemView
from PyQt5.QtGui import QStandardItemModel, QIcon, QFont, QStandardItem
from ..model import action_manager, FileSystem, Controller
from . import ActionButtonGroup, resource_rc


class MainWidget(QWidget):
    #QMainWindow does not allow any self.main_layout or boxes layout. Therefore we use a QWidget instance

    def __init__(self):
        QWidget.__init__(self)
        self.files_system_view = None
        self.all_action_descriptors = []
        self.prefix_action_descriptors = []
        self.extension_action_descriptors = []
        self.file_or_folder = "file"
        self.frame_space = 20
        self.frame_width = 260
        self.frame_height = 290
        self.button_width = 25
        self.init_actions()
        self.init_UI()
        #----------------------------------INIT UI---------------------------------------
        #---INPUTS DEFINITION---
    def init_actions(self):
        original_name_inputs = []
        case_change_inputs = []
        first_letter_inputs = []
        first_letter_inputs.append(action_manager.ActionInput(
            'first_letter', self.tr('First Letter'), "checkable", True))
        first_letter_inputs.append(action_manager.ActionInput(
            'after_symbols', self.tr('And After'), str, "- _"))
        uppercase_inputs = []
        lowercasecase_inputs = []
        character_replacement_inputs = []
        character_replacement_inputs.append(action_manager.ActionInput(
            'old_char', self.tr('Replace'), str, ""))
        character_replacement_inputs.append(action_manager.ActionInput(
            'new_char', self.tr('With'), str, ""))
        character_replacement_inputs.append(action_manager.ActionInput(
            'regex', self.tr('Regex'), "checkable", False))
        character_insertion_inputs = []
        character_insertion_inputs.append(action_manager.ActionInput(
            'new_char', self.tr('Insert'), str, ""))
        character_insertion_inputs.append(action_manager.ActionInput(
            'index', self.tr('At Position'), int, 0))
        character_deletion_inputs = []
        character_deletion_inputs.append(action_manager.ActionInput(
            'starting_position', self.tr('From'), int, 0))
        character_deletion_inputs.append(action_manager.ActionInput(
            'ending_position', self.tr('To'), int, 1))
        custom_name_inputs = []
        custom_name_inputs.append(action_manager.ActionInput(
            'new_name', self.tr('New Name'), str, ""))
        counter_inputs = []
        counter_inputs.append(action_manager.ActionInput(
            'start_index', self.tr('Start At'), int, 0))
        counter_inputs.append(action_manager.ActionInput('increment', self.tr(
            'Increment'), int, 1))
        counter_inputs.append(action_manager.ActionInput(
            'digit_number', self.tr('Number of Digit'), int, 1))
        date_inputs = []
        date_inputs.append(action_manager.ActionInput(
            'is_modified_date', self.tr('Modified'), bool, False))
        date_inputs.append(action_manager.ActionInput(
            'is_created_date', self.tr('Created'), bool, True))
        date_inputs.append(
            action_manager.ActionInput('format_display', self.tr('Format'),
                                       str, "%Y-%m-%d %H:%M:%S (%A %B)"))
        foldername_inputs = []
        image_metadata_inputs = []
        image_date_time_original = []
        image_date_time_original.append(action_manager.ActionInput(
            'time_format', self.tr('Format'), str, "%Y-%m-%d %H:%M:%S"))

        image_f_number = []
        image_exposure_time = []
        image_iso = []
        image_camera_model = []
        image_x_dimension = []
        image_y_dimension = []
        image_focal_length = []
        image_artist = []
        music_metadata_inputs = []
        music_artist = []
        music_title = []
        music_year = []
        music_album = []
        music_genre = []
        music_track = []
        case_action_descriptors = []
        case_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Titlecase"), first_letter_inputs,
            action_manager.TitleCaseAction))
        case_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Uppercase"), uppercase_inputs,
            action_manager.UpperCaseAction))
        case_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Lowercase"), lowercasecase_inputs,
            action_manager.LowerCaseAction))
        image_action_descriptors = []
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Original Date"), image_date_time_original,
            action_manager.ImageDateTimeOriginal))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("F Number"), image_f_number, action_manager.ImageFNumber))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Exposure"), image_exposure_time,
            action_manager.ImageExposureTime))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("ISO"), image_iso, action_manager.ImageISO))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Camera Model"), image_camera_model,
            action_manager.ImageCameraModel))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("X Dimension"), image_x_dimension,
            action_manager.ImageXDimension))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Y Dimension"), image_y_dimension,
            action_manager.ImageYDimension))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Focal Length"), image_focal_length,
            action_manager.ImageFocalLength))
        image_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Artist"), image_artist, action_manager.ImageArtist))
        music_action_descriptors = []
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Artist"), music_artist, action_manager.MusicArtist))
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Title"), music_title, action_manager.MusicTitle))
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Year"), music_year, action_manager.MusicYear))
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Album"), music_album, action_manager.MusicAlbum))
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Genre"), music_genre, action_manager.MusicGenre))
        music_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Track"), music_track, action_manager.MusicTrack))

        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Original Name"), original_name_inputs,
            action_manager.OriginalNameAction))
        self.all_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Case"), case_action_descriptors,
                action_manager.CaseChangeAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Custom Name"), custom_name_inputs,
            action_manager.CustomNameAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Folder Name"), foldername_inputs,
            action_manager.FolderNameUsageAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Find And Replace"), character_replacement_inputs,
            action_manager.CharacterReplacementAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Insert Characters"), character_insertion_inputs,
            action_manager.CharacterInsertionAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Delete Characters"), character_deletion_inputs,
            action_manager.CharacterDeletionAction))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Counter"), counter_inputs, action_manager.Counter))
        self.all_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Date"), date_inputs, action_manager.DateAction))
        self.all_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Image Metadata"), image_action_descriptors,
                action_manager.GenericImageAction))
        self.all_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Music Metadata"), music_action_descriptors,
                action_manager.GenericMusicAction))
        #PREFIX ACTION DESCRIPTOR
        self.prefix_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Custom Name"), custom_name_inputs,
            action_manager.CustomNameAction))
        self.prefix_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Folder Name"), foldername_inputs,
            action_manager.FolderNameUsageAction))
        self.prefix_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Counter"), counter_inputs, action_manager.Counter))
        self.prefix_action_descriptors.append(action_manager.ActionDescriptor(
            self.tr("Date"), date_inputs, action_manager.DateAction))
        self.prefix_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Image Metadata"), image_action_descriptors,
                action_manager.GenericImageAction))
        self.prefix_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Music Metadata"), music_action_descriptors,
                action_manager.GenericMusicAction))

        #EXTENSION ACTION DESCRIPTOR
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptor(
                self.tr("Original Name"), original_name_inputs,
                action_manager.OriginalNameAction))
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptorGroup(
                self.tr("Case"), case_action_descriptors,
                action_manager.CaseChangeAction))
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptor(
                self.tr("Custom Name"), custom_name_inputs,
                action_manager.CustomNameAction))
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptor(
                self.tr("Find And Replace"), character_replacement_inputs,
                action_manager.CharacterReplacementAction))
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptor(
                self.tr("Insert Characters"), character_insertion_inputs,
                action_manager.CharacterInsertionAction))
        self.extension_action_descriptors.append(
            action_manager.ActionDescriptor(
                self.tr("Delete Characters"), character_deletion_inputs,
                action_manager.CharacterDeletionAction))

    def init_UI(self):
        #---LAYOUT---
        self.main_grid = QGridLayout(self)
        self.main_grid.setObjectName("main_grid")
        self.setLayout(self.main_grid)
        #---TREE VIEW---
        self.treeView = QTreeView()
        self.treeView.setStyleSheet(
            "QTreeView{border:2px solid rgb(220, 220, 220); border-radius: 2px};")
        self.treeView.setObjectName("treeView")
        self.treeView.setAlternatingRowColors(True)
        self.treeView.setSortingEnabled(False)
        #self.treeView.setDragDropMode(QAbstractItemView.DropOnly)
        self.model = QStandardItemModel()
        self.model.setObjectName("model")
        self.model.setHorizontalHeaderLabels([self.tr("Original Files"),
                                              self.tr("Modified Files")])
        self.treeView.setModel(self.model)
        self.treeView.setColumnWidth(0, (
            self.treeView.columnWidth(0) + self.treeView.columnWidth(1)) / 2)
        self.main_grid.addWidget(self.treeView, 3, 0)
        self.file_box = ActionButtonGroup(
            self.tr("File"), self.all_action_descriptors, self.frame_width,
            self.frame_height, "file")
        self.file_box.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.file_box.setFixedSize(self.frame_width, self.frame_height)
        self.file_box.changed.connect(self.apply_action)
        self.file_box.addedBefore.connect(self.add_prefix)
        self.file_box.addedAfter.connect(self.add_suffix)
        #---EXTENSION GROUP---
        self.extension_box = ActionButtonGroup(
            self.tr("Extension"), self.extension_action_descriptors,
            self.frame_width, self.frame_height, "extension")
        self.extension_box.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.extension_box.setMinimumSize(self.frame_width, self.frame_height)
        self.extension_box.changed.connect(self.apply_action)
        #Container Widget
        widget_container = QWidget()
        widget_container.setObjectName("widget_container")
        #Layout of Container Widget
        self.scroll_area_layout = QHBoxLayout()
        self.scroll_area_layout.setSpacing(17)
        self.scroll_area_layout.addStretch()
        self.scroll_area_layout.addWidget(self.file_box)
        self.scroll_area_layout.addWidget(self.extension_box)
        self.scroll_area_layout.addStretch()
        widget_container.setLayout(self.scroll_area_layout)
        widget_container.setStyleSheet(
            "QWidget#widget_container{background-color: rgb(226, 226, 226)};")
        #Scroll Area Properties
        scroll = QScrollArea()
        scroll.setObjectName("scroll_area")
        scroll.setAutoFillBackground(False)
        scroll.setStyleSheet(
            "QFrame#scroll_area{border:1px solid rgb(223, 223, 223); border-radius: 2px; padding:2px; background-color: rgb(226, 226, 226)};")
        scroll.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        scroll.setMinimumSize(self.frame_width * 2, self.frame_height + 50)
        scroll.setWidgetResizable(True)
        scroll.setWidget(widget_container)
        #Scroll Area Layer add
        hLayout = QHBoxLayout()
        hLayout.addWidget(scroll)
        font = QFont()
        font.setWeight(75)
        font.setBold(True)
        pattern_label = QLabel(self.tr("Pattern"))
        pattern_label.setFont(font)
        self.main_grid.addWidget(pattern_label, 0, 0)
        self.main_grid.addLayout(hLayout, 1, 0)
        self.folder_icon = QIcon(":/folder_icon.png")
        self.file_icon = QIcon(":/file_icon.png")
        self.preview_label = QLabel(self.tr("Preview"))
        self.preview_label.setFont(font)
        self.main_grid.addWidget(self.preview_label, 2, 0)

    def add_prefix(self, widget_caller):
        self.prefix_box = ActionButtonGroup(
            self.tr("Prefix"), self.prefix_action_descriptors,
            self.frame_width, self.frame_height, "prefix")
        self.prefix_box.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.prefix_box.setMinimumSize(self.frame_width, self.frame_height)
        self.prefix_box.addedBefore.connect(self.add_prefix)
        self.prefix_box.removed.connect(self.remove_widget)
        self.prefix_box.changed.connect(self.apply_action)
        self.scroll_area_layout.insertWidget(
            self.scroll_area_layout.indexOf(widget_caller), self.prefix_box)
        self.apply_action()

    def add_suffix(self, widget_caller):
        self.suffix_box = ActionButtonGroup(
            self.tr("Suffix"), self.prefix_action_descriptors,
            self.frame_width, self.frame_height, "suffix")
        self.suffix_box.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.suffix_box.setMinimumSize(self.frame_width, self.frame_height)
        self.suffix_box.addedAfter.connect(self.add_suffix)
        self.suffix_box.removed.connect(self.remove_widget)
        self.suffix_box.changed.connect(self.apply_action)
        self.scroll_area_layout.insertWidget(
            self.scroll_area_layout.indexOf(widget_caller) + 1,
            self.suffix_box)
        self.apply_action()

    def remove_widget(self, widget_caller):
        self.scroll_area_layout.removeWidget(widget_caller)
        widget_caller.destruct_layout()
        self.apply_action()

    def is_file(self, value):
        if value is True:
            self.file_or_folder = "file"
            self.file_box.set_label(self.tr("File"))
            self.file_box.set_frame_type("file")
            self.extension_box.show()
        else:
            self.file_or_folder = "folder"
            self.extension_box.hide()
            self.file_box.set_label(self.tr("Folder"))
            self.file_box.set_frame_type("folder")
        self.apply_action()

    def get_action_button_group(self):
        return self.file_box

    def set_filtered_files(self, files_system_view):
        """Process the selected directory to create the tree and modify the files"""
        self.files_system_view = files_system_view
        self.controller = Controller(self.files_system_view)
        self.redraw_tree()
        self.apply_action()

    def redraw_tree(self):
        tree = self.main_grid.itemAtPosition(2, 0)
        self.model.clear()
        self.model.setHorizontalHeaderLabels([self.tr("Original Files"),
                                              self.tr("Modified Files")])
        root_tree_node_view = self.files_system_view.get_file_system_tree_node(
        )
        self.preview_label.setText(self.tr("Preview") + ": " +
                                   root_tree_node_view.get_original_path())
        self.populate_tree(self.model, root_tree_node_view, True)
        self.treeView.setColumnWidth(0, (
            self.treeView.columnWidth(0) + self.treeView.columnWidth(1)) / 2)
        self.treeView.expandAll()

    def populate_tree(self, parent, tree_node, reset_view):
        """Populate the tree with the selected directory. If reset_view is False, only the modified_files are updated."""
        children = tree_node.get_children()
        for i, child in enumerate(children):
            if child.original_filedescriptor.is_folder:
                icon = self.folder_icon
            else:
                icon = self.file_icon
            original_file = QStandardItem(
                icon, child.original_filedescriptor.basename)
            original_file.setEditable(False)
            modified_file = QStandardItem(
                child.modified_filedescriptor.basename)
            modified_file.setEditable(False)
            if isinstance(parent, QStandardItemModel):
                #parent.itemChanged[QStandardItem].connect(self.tree_item_changed)
                if reset_view:
                    parent.setItem(i, 0, original_file)
                parent.setItem(i, 1, modified_file)
                self.populate_tree(parent.item(i, 0), child, reset_view)
            else:
                if reset_view:
                    parent.setChild(i, 0, original_file)
                parent.setChild(i, 1, modified_file)
                self.populate_tree(parent.child(i, 0), child, reset_view)
    #@Slot()            
    #def tree_item_changed(self, selected_item):
    #    pass
    #    #print(selected_item.row())

    def rename(self):
        """Rename all the files and folders."""
        try:
            self.controller.batch_rename()
        except Exception as e:
            QMessageBox.critical(self, "Error", str(e))
        # self.populate_tree(self.model, self.files_system_view.get_file_system_tree_node(), True)
        # self.apply_action()

    def undo(self):
        """Undo the previous renaming action."""
        self.controller.batch_undo()
        self.populate_tree(self.model,
                           self.files_system_view.get_file_system_tree_node(),
                           True)
        self.apply_action()

    def apply_action(self):
        self.actions = []
        widget_number = self.scroll_area_layout.count()
        for i in range(1, widget_number - 1):  #do not count the stretch widget
            action_button_group = self.scroll_area_layout.itemAt(i).widget()
            self.actions.append(action_button_group.get_action_with_inputs())
        if self.files_system_view is not None:
            try:
                self.controller.batch_update(self.actions, self.file_or_folder)
            except Exception as e:
                QMessageBox.warning(self, "Warning", str(e))
            #refresh tree
            self.populate_tree(
                self.model, self.files_system_view.get_file_system_tree_node(),
                False)


class SizeCalculator(object):
    def __init__(self, main_window):
        self.main_window = main_window
        main_widget = main_window.get_main_widget()
        action_button_group = main_widget.get_action_button_group()
        for i in range(len(action_button_group.get_action_descriptors())):
            action_button_group.on_selected_action_changed(i)
            self.max_height = action_button_group.get_maximum_height()
            self.max_width = action_button_group.get_maximum_width()
        return (self.max_height, self.max_width)
